@@ -11,22 +11,22 @@ Notes:
     this part are one copy from deap,
     change the random to numpy.random
 """
+
 import copy
 import operator
 import sys
 from collections import Counter
 from functools import wraps
 from inspect import isclass
+from operator import attrgetter
 
+import numpy as np
 from deap.tools import Statistics, MultiStatistics
 from numpy import random
-from operator import attrgetter
-from featurebox.symbol.dim import Dim, dnan
-import numpy as np
+
+from featurebox.symbol.calculation.scores import score_dim
 
 
-# from featurebox.tools.tool import logg
-# from featurebox.tools.tool import logg
 ######################################
 # Generate                         #
 ######################################
@@ -48,12 +48,31 @@ def checkss(func):
     return wrapper
 
 
+def checks_number(func):
+    @wraps(func)
+    def wrapper(*args, **kwargs):
+        result = func(*args, **kwargs)
+        if "force_number" in kwargs:
+            fs = kwargs["force_number"]
+        else:
+            fs = False
+        if fs and not len(result):
+            raise TypeError("The DimForceLoop keep the dim of the select offspring,\n"
+                            "But the select number from last population are zero at the current dim_type limitation.\n"
+                            "Please change the dim_type or change the DimForceLoop method to others")
+
+        return result
+
+    return wrapper
+
+
 def generate(pset, min_, max_, condition, personal_map=False, *kwargs):
     """
 
     Parameters
     ----------
     pset: SymbolSet
+        pset
     min_: int
         Minimum height of the produced trees.
     max_: int
@@ -65,6 +84,7 @@ def generate(pset, min_, max_, condition, personal_map=False, *kwargs):
     kwargs: None
         placeholder for future
     personal_map:bool
+        premap
 
     Returns
     -------
@@ -111,11 +131,16 @@ def generate(pset, min_, max_, condition, personal_map=False, *kwargs):
 
     dispose = list(random.choice(pset.dispose, len(expr), p=pset.prob_dispose_list))
 
-    add_ = [i for i in pset.dispose if i.name == "MAdd"]
-    mul_ = [i for i in pset.dispose if i.name == "MMul"]
-    add_.extend(mul_)
+    if pset.types == 1:
+        add_ = [pset.dispose_dict["Self"]]
+        dispose = add_*len(expr)
 
-    dispose[0] = random.choice(add_, p=[0.8, 0.2])
+    elif pset.types == 2:
+        add_ = list(pset.dispose_dict[i] for i in ["MAdd", "MSub", "MMul", "MDiv"] )
+        dispose[0] = random.choice(add_, p=[0.25, 0.25, 0.25, 0.25])
+    else:
+        add_ = list(pset.dispose_dict[i] for i in ["MAdd", "MMul"] )
+        dispose[0] = random.choice(add_, p=[0.5, 0.5])
 
     re = []
     for i, j in zip(dispose, expr):
@@ -131,7 +156,8 @@ def genGrow(pset, min_, max_, personal_map=False, ):
     :param pset: Primitive set from which primitives are selected.
     :param min_: Minimum height of the produced trees.
     :param max_: Maximum Height of the produced trees.
-    :param personal_map: bool
+    :param personal_map: bool.
+    
     :returns: A grown tree with leaves at possibly different depths.
 
     """
@@ -145,64 +171,6 @@ def genGrow(pset, min_, max_, personal_map=False, ):
     return generate(pset, min_, max_, condition, personal_map=personal_map)
 
 
-def compile_(expr, pset):
-    """Compile the expression *expr*.
-
-    :param expr: Expression to compile. It can either be a PrimitiveTree,
-                 a string of Python code or any object that when
-                 converted into string produced a valid Python code
-                 expression.
-    :param pset: Primitive set against which the expression is compile.
-    :returns: a function if the primitive set has 1 or more arguments,
-              or return the results produced by evaluating the tree.
-    """
-    if isinstance(expr, str):
-        code = expr
-    else:
-        code = repr(expr)
-    if len(pset.arguments) > 0:
-        # This section is a stripped version of the lambdify
-        # function of SymPy 0.6.6.
-        args = ",".join(arg for arg in pset.arguments)
-        code = "lambda {args}: {code}".format(args=args, code=code)
-    try:
-        return eval(code, pset.context, {})
-    except MemoryError:
-        _, _, traceback = sys.exc_info()
-        raise MemoryError("DEAP : Error in tree evaluation :"
-                          " Python cannot evaluate a tree higher than 90. "
-                          "To avoid this problem, you should use bloat control on your "
-                          "operators. See the DEAP documentation for more information. "
-                          "DEAP will now abort.").with_traceback(traceback)
-
-
-def compile_context(expr, context):
-    """Compile the expression *expr*.
-
-    :param expr: Expression to compile. It can either be a PrimitiveTree,
-                 a string of Python code or any object that when
-                 converted into string produced a valid Python code
-                 expression.
-    :param context: dict
-    :returns: a function if the primitive set has 1 or more arguments,
-              or return the results produced by evaluating the tree.
-    """
-    if isinstance(expr, str):
-        code = expr
-    else:
-        code = repr(expr)
-
-    try:
-        return eval(code, context, {})
-    except MemoryError:
-        _, _, traceback = sys.exc_info()
-        raise MemoryError("DEAP : Error in tree evaluation :"
-                          " Python cannot evaluate a tree higher than 90. "
-                          "To avoid this problem, you should use bloat control on your "
-                          "operators. See the DEAP documentation for more information. "
-                          "DEAP will now abort.").with_traceback(traceback)
-
-
 def depart(individual):
     if len(individual) <= 10 or individual.height <= 8:
         return [individual, ]
@@ -281,7 +249,9 @@ def staticLimit(key, max_value):
             new_inds = list(func(*args, **kwargs))
             for i, ind in enumerate(new_inds):
 
-                if key(ind) > max_value:
+                if key(ind) == key(keep_inds[i]):
+                    pass
+                elif key(ind) > max_value:
                     new_inds[i] = keep_inds[random.choice(len(keep_inds))]
 
             return new_inds
@@ -317,107 +287,6 @@ def mutUniform(individual, expr, pset):
     return individual,
 
 
-# @logg
-# @checkss
-def mutNodeReplacement(individual, pset):
-    """Replaces a randomly chosen primitive from *individual* by a randomly
-    chosen primitive with the same number of arguments from the :attr:`pset`
-    attribute of the individual.
-
-    :param individual: The normal or typed tree to be mutated.
-    :param pset: SymbolSet
-    :returns: A tuple of one tree.
-    """
-
-    if len(individual) < 4:
-        return individual,
-
-    if random.random() <= 0.8:
-        index = random.choice(np.arange(1, len(individual), step=2))
-    else:
-        index = random.choice(np.arange(0, len(individual), step=2))
-
-    node = individual[index]
-
-    if index % 2 == 0:
-        for i in pset.dispose:
-            assert i.arity == 1
-        prims = pset.dispose
-        p_d = np.array([pset.prob_dispose[repr(i)] for i in prims], 'float32')
-        p_d /= np.sum(p_d)
-        a = prims[random.choice(len(prims), p=p_d)]
-        individual[index] = a
-    else:
-
-        if node.arity == 0:  # Terminal
-            p_t = pset.prob_ter_con_list
-            term = pset.terminals_and_constants[random.choice(len(pset.terminals_and_constants), p=p_t)]
-            individual[index] = term
-        else:  # Primitive
-            prims = [p for p in pset.primitives if p.arity == node.arity]
-            p_p = np.array([pset.prob_pri[repr(i)] for i in prims], 'float32')
-
-            p_p /= np.sum(p_p)
-            # except:
-            a = prims[random.choice(len(prims), p=p_p)]
-
-            individual[index] = a
-
-    return individual,
-
-
-# @logg
-# @checkss
-def mutDifferentReplacement(individual, pset):
-    """Replaces a randomly chosen primitive from *individual* by a randomly
-    chosen primitive with the same number of arguments from the :attr:`pset`
-    attribute of the individual.
-    decrease the probability of same terminals.
-
-    :param individual: The normal or typed tree to be mutated.
-    :param pset: SymbolSet
-    :returns: A tuple of one tree.
-    """
-    if len(individual) < 4:
-        return individual,
-
-    ters = [repr(i) for i in individual.terminals()]
-    pset_ters = [repr(i) for i in pset.terminals_and_constants]
-
-    cou = Counter(ters)
-    cou_mutil = {i: j for i, j in cou.items() if j >= 2}
-    ks = list(cou_mutil.keys())
-    nks = list(set(pset_ters) - (set(ks)))
-
-    p_nks = np.array([pset.prob_ter_con[i] for i in nks])
-    p_nks /= np.sum(p_nks)
-
-    if cou_mutil:
-        indexs = []
-        for k, v in cou_mutil.items():
-            indi = []
-            for i in np.arange(1, len(individual), 2):
-                if repr(individual[i]) == k:
-                    indi.append(i)
-            if indi:
-                indexs.append(random.choice(indi))
-        if len(indexs) <= len(nks):
-            term = random.choice(nks, len(indexs), replace=False, p=p_nks)
-        else:
-            term = random.choice(nks, len(indexs), replace=True, p=p_nks)
-
-        term_ters = []
-        for name in term:
-            for i in pset.terminals_and_constants:
-                if repr(i) == name:
-                    term_ters.append(i)
-
-        for o, n in zip(indexs, term_ters):
-            individual[o] = n
-
-    return individual,
-
-
 # @logg
 # @checkss
 def mutShrink(individual, pset=None):
@@ -430,7 +299,7 @@ def mutShrink(individual, pset=None):
     """
     _ = pset
     # We don't want to "shrink" the root
-    if len(individual) < 6 or individual.height <= 4:
+    if len(individual) < 4 or individual.height <= 4:
         return individual,
 
     index = random.randint(0, len(individual))
@@ -468,13 +337,16 @@ def mutNodeReplacementVerbose(individual, pset, personal_map=False):
     if len(individual) < 4:
         return individual,
 
-    if random.random() <= 0.8:
-        index = random.choice(np.arange(1, len(individual), step=2))
+    if pset.types > 1:
+        if random.random() <= 0.8:
+            index = random.choice(np.arange(1, len(individual), step=2))
+        else:
+            index = random.choice(np.arange(0, len(individual), step=2))
     else:
         index = random.choice(np.arange(0, len(individual), step=2))
-
     node = individual[index]
 
+
     if index % 2 == 0:
         for i in pset.dispose:
             assert i.arity == 1
@@ -492,7 +364,9 @@ def mutNodeReplacementVerbose(individual, pset, personal_map=False):
                     p_t = pset.prob_ter_con_list
             else:
                 p_t = pset.prob_ter_con_list
+
             term = pset.terminals_and_constants[random.choice(len(pset.terminals_and_constants), p=p_t)]
+
             individual[index] = term
         else:  # Primitive
             prims = [p for p in pset.primitives if p.arity == node.arity]
@@ -525,14 +399,13 @@ def mutDifferentReplacementVerbose(individual, pset, personal_map=False):
     """
     if len(individual) < 4:
         return individual,
-
     ters = [repr(i) for i in individual.terminals()]
     pset_ters = [repr(i) for i in pset.terminals_and_constants]
-
     cou = Counter(ters)
     cou_mutil = {i: j for i, j in cou.items() if j >= 2}
     ks = list(cou_mutil.keys())
     nks = list(set(pset_ters) - (set(ks)))
+    nks.sort()  # very import for random
 
     p_nks = np.array([pset.prob_ter_con[i] for i in nks])
     p_nks /= np.sum(p_nks)
@@ -621,10 +494,12 @@ def selTournament(individuals, k, tournsize, fit_attr="fitness"):
     return chosen
 
 
-def selKbestDim(pop, K_best=10, dim_type=None, fuzzy=False, fit_attr="fitness"):
+@checks_number
+def selKbestDim(pop, K_best=10, dim_type=None, fuzzy=False, fit_attr="fitness", force_number=False):
     """
-    select the individual with dim limitation.
+    Select the individual with dim limitation.
     Parameters
+    
     ----------
     pop: SymbolTree
         A list of individuals to select from.
@@ -635,6 +510,11 @@ def selKbestDim(pop, K_best=10, dim_type=None, fuzzy=False, fit_attr="fitness"):
         the dim or the dim with same base. such as m,m^2,m^3
     fit_attr:str
         The attribute of individuals to use as selection criterion
+    force_number:
+        return the number the same with K
+    .. note::
+        default attr is "fitness"
+        
     Returns
     -------
     A list of selected individuals.
@@ -642,56 +522,66 @@ def selKbestDim(pop, K_best=10, dim_type=None, fuzzy=False, fit_attr="fitness"):
     chosen = sorted(pop, key=operator.attrgetter(fit_attr))
     chosen.reverse()
 
-    if dim_type is None:
-        add_ind = chosen
-    elif dim_type is 'integer':
-        add_ind = [ind for ind in chosen if ind.y_dim.isinteger]
-    elif dim_type is 'coef':
-        add_ind = [ind for ind in chosen if not ind.y_dim.anyisnan()]
-    elif isinstance(dim_type, list):
-        add_ind = [ind for ind in chosen if ind.y_dim in dim_type]
-    elif isinstance(dim_type, Dim):
-        if fuzzy:
-            add_ind = [ind for ind in chosen if ind.y_dim.is_same_base(dim_type)]
-        else:
-            add_ind = [ind for ind in chosen if ind.y_dim == dim_type]
-    else:
-        raise TypeError("dim_type should be None, 'integer', special Dim or list of Dim")
+    choice_index = [score_dim(ind.y_dim, dim_type, fuzzy) for ind in chosen]
+    add_ind = [chosen[i] for i, j in enumerate(choice_index) if j == 1]
+
     if K_best is None:
         if len(add_ind) >= 5:
-            K_best = round(len(add_ind) / 10)
+            K_best = round(len(pop) / 10)
         else:
             K_best = 0
     if len(add_ind) >= round(K_best):
         return add_ind[:round(K_best)]
     else:
-        return add_ind
+        if not force_number or len(add_ind) == 0:
+            return add_ind
+        else:
+            ti = K_best // len(add_ind)
+            yu = K_best % len(add_ind)
+            add_new = []
+            for i in range(ti):
+                add_new.extend(add_ind)
+            add_new.extend(add_ind[:yu])
+            return add_new
 
 
 def Statis_func(stats=None):
     if stats is None:
-        stats = {"fitness_dim_is_target_max": ("max",), "dim_is_traget": ("sum",)}
+        stats = {"fitness_dim_max": ("max",), "dim_is_target": ("sum",)}
 
     func = {"max": np.max, "mean": np.mean, "min": np.mean, "std": np.std, "sum": np.sum}
-    att = {"fitness": lambda ind: ind.fitness.values[0],
-           "fitness_dim_is_True_max": lambda ind: ind.fitness.values[0] if ind.y_dim is not dnan else -np.inf,
-           "fitness_dim_is_target_max": lambda ind: ind.fitness.values[0] if ind.dim_score else -np.inf,
-           "fitness_dim_is_True_min": lambda ind: ind.fitness.values[0] if ind.y_dim is not dnan else np.inf,
-           "fitness_dim_is_target_min": lambda ind: ind.fitness.values[0] if ind.dim_score else np.inf,
-           "dim_is_True": lambda ind: 1 if ind.y_dim is not dnan else 0,
-           "dim_is_traget": lambda ind: 1 if ind.dim_score else 0}
+    att = {
+
+        "fitness": lambda ind: ind.fitness.values[0],
+        "fitness_dim_max": lambda ind: ind.fitness.values[0] if ind.dim_score else -np.inf,
+        "fitness_dim_min": lambda ind: ind.fitness.values[0] if ind.dim_score else np.inf,
+        "dim_is_target": lambda ind: 1 if ind.dim_score else 0,
+        # special
+        "coef": lambda ind: score_dim(ind.y_dim, "coef", fuzzy=False),
+        "integer": lambda ind: score_dim(ind.y_dim, "integer", fuzzy=False),
+
+        "length": lambda ind: len(ind),
+        "height": lambda ind: ind.height,
+
+        # mutil-target
+        "weight_fitness": lambda ind: ind.fitness.wvalues,
+        "weight_fitness_dim": lambda ind: ind.fitness.wvalues if ind.dim_score else -np.inf,
+        # weight have mul the "-"
+    }
 
     sa_all = {}
 
     for a, f in stats.items():
         if a in att:
             a_s = att[a]
-        else:
+        elif isinstance(callable, a):
             a_s = a
             a = str(a).split(" ")[1]
-
+        else:
+            raise TypeError("the key must be in definition or a function")
         sa = Statistics(a_s)
-
+        if isinstance(f, str):
+            f = [f, ]
         for i, fi in enumerate(f):
             assert fi in func
             ff = func[fi]
@@ -801,6 +691,7 @@ def varAnd(population, toolbox, cxpb, mutpb):
 
 def varAndfus(population, toolbox, cxpb, mutpb, fus):
     offspring = copy.deepcopy(population)
+
     # Apply crossover and mutation on the offspring
     for i in range(1, len(offspring), 2):
         if random.random() < cxpb:
@@ -813,7 +704,8 @@ def varAndfus(population, toolbox, cxpb, mutpb, fus):
     for j in fus:
         for i in range(len(offspring)):
             if random.random() < mutpb:
-                offspring[i], = j(individual=offspring[i])
+                # print(random.random(), i)
+                offspring[i], = j(offspring[i])
                 del offspring[i].fitness.values
 
     return offspring
