@@ -1,8 +1,7 @@
 import unittest
 import numpy as np
 
-from featurebox.symbol.dim import Dim, dless, dnan, dim_map
-from featurebox.symbol.preference import PreMap
+from featurebox.symbol.probability.preference import PreMap
 
 
 class MyTestCase(unittest.TestCase):
